package org.funtester.app.util.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * A panel that can be used in CRUD operations.
 * 
 * @author Thiago Delgado Pinto
 *
 */
public class CRUDListPanel extends JPanel {

	private static final long serialVersionUID = 2236509248871913586L;

	private boolean editable = true;
	
	private AbstractAction newAction;
	private AbstractAction editAction;
	private AbstractAction removeAction;
	
	protected JPanel buttonsPanel;
	protected JButton newButton;
	protected JButton editButton;
	protected JButton removeButton;
	
	private JTable table;
	private TableModel tableModel;	

	public CRUDListPanel() {		
		setLayout(new FormLayout(new ColumnSpec[] {
				ColumnSpec.decode("default:grow"),},
			new RowSpec[] {
				FormFactory.RELATED_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC,
				FormFactory.RELATED_GAP_ROWSPEC,
				RowSpec.decode("default:grow"),}));
		
		buttonsPanel = new JPanel();
		buttonsPanel.setName("buttonsPanel");
		add(buttonsPanel, "1, 2, fill, default");
		buttonsPanel.setLayout(new FormLayout(new ColumnSpec[] {
				FormFactory.BUTTON_COLSPEC,
				FormFactory.RELATED_GAP_COLSPEC,
				FormFactory.BUTTON_COLSPEC,
				FormFactory.RELATED_GAP_COLSPEC,
				FormFactory.BUTTON_COLSPEC,},
			new RowSpec[] {
				FormFactory.DEFAULT_ROWSPEC,}));		
		
		newButton = new JButton( newAction );
		newButton.setName( "newButton" );
		buttonsPanel.add(newButton, "1, 1");
		
		editButton = new JButton( editAction );
		editButton.setName( "editButton" );
		buttonsPanel.add(editButton, "3, 1");
		
		removeButton = new JButton( removeAction );
		removeButton.setName( "removeButton" );
		buttonsPanel.add(removeButton, "5, 1");
		
		JPanel listPanel = new JPanel();
		listPanel.setName("listPanel");
		listPanel.setLayout(new BorderLayout(0, 0));
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setMinimumSize(new Dimension(300, 50));
		listPanel.add(scrollPane, BorderLayout.CENTER);
		
		table = new JTable();
		table.setName("table");
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		// Update Edit and Remove enabled status
		table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {					
					@Override
					public void valueChanged(ListSelectionEvent e) {
						if ( e.getValueIsAdjusting() ) {
							return;
						}
						updateActionsEnabledStatus( table.getSelectedRow() );
					}
				} );
		
		table.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if ( MouseEvent.BUTTON1 == e.getButton()
						&& 2 == e.getClickCount()
						&& getEditAction() != null
						&& getEditAction().isEnabled() ) {
					getEditAction().actionPerformed( null );
				}
			}
		});		
		
		scrollPane.setViewportView(table);		

		listPanel.setPreferredSize(new Dimension(452, 50));
		add(listPanel, "1, 4, fill, fill");
		
		updateActionsEnabledStatus( -1 ); // Start disabled
	}
	
	protected void updateActionsEnabledStatus(int index) {
		if ( getEditAction() != null ) {
			getEditAction().setEnabled( editable && index >= 0 );
		}
		
		if ( getRemoveAction() != null ) {
			getRemoveAction().setEnabled( editable && index >= 0 );
		}
	}

	public JTable getTable() {
		return table;
	}
	
	public void setEditable(boolean editable) {
		this.editable = editable;
		buttonsPanel.setEnabled( editable );
		buttonsPanel.setVisible( editable );
		newAction.setEnabled( editable );
		editAction.setEnabled( editable );
		removeAction.setEnabled( editable );
	}
	
	public AbstractAction getNewAction() {
		return newAction;
	}
	
	public void setNewAction(AbstractAction action) {
		this.newAction = action;
		newButton.setAction( this.newAction );
	}
	
	public AbstractAction getEditAction() {
		return editAction;
	}
	
	public void setEditAction(AbstractAction action) {
		this.editAction = action;
		this.editAction.setEnabled( getTable().getSelectedRow() >= 0 );
		editButton.setAction( this.editAction );
	}
	
	public AbstractAction getRemoveAction() {
		return removeAction;
	}
	
	public void setRemoveAction(AbstractAction action) {
		this.removeAction = action;
		this.removeAction.setEnabled( getTable().getSelectedRow() >= 0 );
		removeButton.setAction( this.removeAction );
	}
	
	public void setTableModel(TableModel model) {
		this.tableModel = model;
		table.setModel( this.tableModel );
	}
	
	public void addListSelectionListener(ListSelectionListener listener) {
		table.getSelectionModel().addListSelectionListener(	listener );
	}
}
